package Debian::Defoma::Common;

use strict;
use POSIX;
use Exporter;

use vars qw(@EXPORT @EXPORT_OK @ISA $ROOTDIR $DEFOMA_TEST_DIR $SCRIPTDIR
	    $SUBSTRULEDIR $DEFAULT_PACKAGE $DEFAULT_CATEGORY
	    $QUIET $ERROR $VERBOSE $DEBUG);

@ISA = qw(Exporter);
@EXPORT = qw(&printm &printw &printe &printv &printd &get_files &arg_check
	     &add_hash_list &parse_hints_start &parse_hints_cut
	     &parse_hints_cut_except &parse_hints_build
	     $DEFOMA_TEST_DIR $ROOTDIR $SCRIPTDIR $SUBSTRULEDIR $ERROR
	     $DEFAULT_PACKAGE $DEFAULT_CATEGORY);
@EXPORT_OK = qw(&mylock &register_id_object &register_subst_object
		&get_id_object &get_subst_object &parse_option);
		

$DEFOMA_TEST_DIR = exists($ENV{'DEFOMA_TEST_DIR'}) ?
    $ENV{'DEFOMA_TEST_DIR'} : '';
$ROOTDIR = "$DEFOMA_TEST_DIR/var/lib/defoma";
$SCRIPTDIR = "$DEFOMA_TEST_DIR/usr/share/defoma/scripts";
$SUBSTRULEDIR = "$DEFOMA_TEST_DIR/etc/defoma";

$DEFAULT_PACKAGE = $DEFAULT_CATEGORY = '';

$QUIET = 0;
$ERROR = 1;
$VERBOSE = 0;
$DEBUG = 0;

sub parse_option {
    while (@ARGV > 0 && $ARGV[0] =~ /^-/) {
	my $options = shift(@ARGV);
	$options =~ s/^-//;

	while ($options ne '') {
	    $options =~ s/(.)(.*)/$2/;
	    my $option = $1;

	    $QUIET = 1 if ($option eq 'q');
	    $ERROR = 0 if ($option eq 't');
	    $VERBOSE = 1 if ($option eq 'v');
	    $DEBUG = 1 if ($option eq 'd');
	}
    }
}

sub printd {
    return unless ($DEBUG);

    print STDERR @_, "\n";
}

sub printm {
    return if ($QUIET);

    print STDERR @_, "\n";
}

sub printw {
    my $a = $0;
    $a =~ /^(.*)\/(.*)$/;
    
    print STDERR "$a: ", @_, "\n";
}

sub printe {
    printw @_;

    kill('TERM', $$);
}

sub printv {
    return unless ($VERBOSE);

    print @_, "\n";
}

sub get_files {
    my $pattern = shift;
    my $directory = shift;
    my $i;
    my @caches = ();
    my @list;

    opendir(D, $directory) || return -1;
    @list = readdir(D);
    closedir(D);

    foreach $i (@list) {
	if ($i =~ /$pattern/) {
	    push(@caches, $i);
	}
    }

    return @caches;
}

sub arg_check {
    while (@_ > 0) {
	my $s = shift;
	if ($s =~ /[ \t]/ || $s eq '') {
	    printw "$s: Illegal argument.";
	    return 0;
	}
    }

    return 1;
}

sub add_hash_list {
    my $hashptr = shift;
    my $key = shift;
    my $str = shift;

    if (exists($hashptr->{$key})) {
	$hashptr->{$key} .= ' ';
    } else {
	$hashptr->{$key} = '';
    }

    $hashptr->{$key} .= $str;
}

sub mylock {
    return;
    
    my $lockfile = "$DEFOMA_TEST_DIR/var/lock/defoma";
    my $flag = 0;
    my $op = shift;
    my $bg = (@_ > 0) ? shift(@_) : '';

    if ($op == 0) {
	unlink($lockfile);
    } elsif ($op == 1) {
	while (1) {
	    symlink("locked", $lockfile) && last;

	    if ($flag == 0 && $QUIET == 0) {
		if ($bg) {
		    printm("Waiting for another defoma process to finish...");
		    $flag = 1;
		} else {
		    printe("Another defoma process seems running.");
		}
	    }
	    sleep 10;
	}
    }
}

my %IdObject = ();

sub register_id_object {
    my $o = shift;
    my $pkg = shift;
    my $suffix = shift;

    $IdObject{"$pkg/$suffix"} = $o;
}

sub get_id_object {
    my $pkg = shift;
    my $suffix = shift;

    if (exists($IdObject{"$pkg/$suffix"})) {
	return $IdObject{"$pkg/$suffix"};
    }

    return '';
}

my %SubstObject = ();

sub register_subst_object {
    my $o = shift;
    my $rulename = shift;

    $SubstObject{$rulename} = $o;
}

sub get_subst_object {
    my $rulename = shift;

    if (exists($SubstObject{$rulename})) {
	return $SubstObject{$rulename};
    }

    return '';
}

sub parse_hints_start {
    my $ret = {};
    my $key = '';
    my $addflag = 0;

    foreach my $item (@_) {
	if ($item =~ /^--(.*)/) {
	    if ($key && $addflag == 0) {
		$ret->{$key} = '';
	    }
	    $key = $1;
	    $addflag = 0;
	} elsif ($key) {
	    $addflag = 1;
	    add_hash_list($ret, $key, $item);
	}
    }

    if ($key && $addflag == 0) {
	$ret->{$key} = '';
    }

    return $ret;
}

sub parse_hints_cut {
    my $parsed = shift;
    my $key;

    foreach $key (@_) {
	if (exists($parsed->{$key})) {
	    delete($parsed->{$key});
	}
    }
}

sub parse_hints_cut_except {
    my $parsed = shift;
    my $key;
    my @l = keys(%{$parsed});

    foreach $key (@l) {
	unless (grep($_ eq $key, @_)) {
	    delete($parsed->{$key});
	}
    }
}

sub parse_hints_build {
    my $parsed = shift;
    my $key;
    my @keys = keys(%{$parsed});
    my @ret = ();

    foreach $key (@keys) {
	push(@ret, '--' . $key);
	push(@ret, split(/ +/, $parsed->{$key}));
    }

    return @ret;
}

package Debian::Defoma::Configure;
use strict;
use POSIX;

use Debian::Defoma::Common;
import Debian::Defoma::Common;

use vars qw(@ISA $SCRIPTDIR $DEFAULT_PACKAGE $DEFAULT_CATEGORY
	    @ACCEPT_CATEGORIES);

my %Initialized = ();
my %Categories = ();
my %AppCategories = ();
my @Scripts = ();
my @DefaultPackage = ();
my @DefaultCategory = ();

sub init {
    return unless (-d $SCRIPTDIR);
    
    @Scripts = get_files("\.defoma\$", $SCRIPTDIR);

    foreach my $script (@Scripts) {
	@ACCEPT_CATEGORIES = ();
	require("$SCRIPTDIR/$script");
	
	$script =~ s/\.defoma$//;

	if (@ACCEPT_CATEGORIES > 0) {
	    foreach my $i (@ACCEPT_CATEGORIES) {
		$Categories{$i} = [] unless (exists($Categories{$i}));
		push(@{$Categories{$i}}, $script);
	    }
	    $AppCategories{$script} = [];
	    push(@{$AppCategories{$script}}, @ACCEPT_CATEGORIES);
	}
    }
}

sub call_1 {
    my $fobj = shift;
    my $app = shift;
    my $com = shift;
    my $category = shift;
    my $font = shift;

    if ($fobj && $com eq 'unregister') {
	if ($fobj->remove_failed($font, $app)) {
	    return 0;
	}
    }

    push(@DefaultPackage, $DEFAULT_PACKAGE);
    push(@DefaultCategory, $DEFAULT_CATEGORY);

    $DEFAULT_PACKAGE = $app;
    $DEFAULT_CATEGORY = $category;

    my $appi = $app;
    $appi =~ s/[^a-zA-Z0-9]/_/g;
    my $ctgi = $category;
    $ctgi =~ s/[^a-zA-Z0-9]/_/g;

    unless (exists($Initialized{"$category/$app"})) {
	$Initialized{"$category/$app"} = '';
	eval("${appi}::${ctgi}('init')");
	printw($@) if ($@);
    }

    my $ret = eval("${appi}::${ctgi}(\$com, \$font, \@_)");
    printw($@) if ($@);

    $DEFAULT_PACKAGE = pop(@DefaultPackage);
    $DEFAULT_CATEGORY = pop(@DefaultCategory);

    if ($ret && $fobj && $com eq 'register') {
	$fobj->add_failed($font, $app, $ret);
	printv("$font: failed to register for package $app, status($ret).");
    }

    return $ret;
}

sub call_m {
    my $fobj = shift;
    my $com = shift;
    my $category = shift;
    my $font = shift;

    return unless (exists($Categories{$category}));

    foreach my $app (@{$Categories{$category}}) {
	call_1($fobj, $app, $com, $category, $font, @_);
    }

    return 0;
}

sub term {
    my @list = keys(%Initialized);
    my ($i, $c, $a);
    
    foreach $i (@list) {
	$i =~ /(.*)\/(.*)/;
	$c = $1;
	$a = $2;
	
	push(@DefaultPackage, $DEFAULT_PACKAGE);
	push(@DefaultCategory, $DEFAULT_CATEGORY);
	
	$DEFAULT_PACKAGE = $a;
	$DEFAULT_CATEGORY = $c;

	$a =~ s/[^a-zA-Z0-9]/_/g;
	$c =~ s/[^a-zA-Z0-9]/_/g;
	
	eval("${a}::${c}('term')");
	printw($@) if ($@);

	$DEFAULT_PACKAGE = pop(@DefaultPackage);
	$DEFAULT_CATEGORY = pop(@DefaultCategory);
    }

    return 0;
}

sub get_app_categories {
    my $app = shift;

    return () unless (exists($AppCategories{$app}));
    return @{$AppCategories{$app}};
}

1;    
